/*****************************************************************
/
/ File   :   ifdhandler.c
/ Author :   David Corcoran <corcoran@linuxnet.com>
/ Date   :   June 15, 2000
/ Purpose:   This provides reader specific low-level calls.
/            See http://www.linuxnet.com for more information.
/ License:   See file LICENSE
/
******************************************************************/

#include <pcscdefines.h>
#include <LinuxDefines.h>
#include <ifdhandler.h>

static struct _IFDCard {
  PUCHAR Atr[MAX_ATR_SIZE];
  DWORD  AtrLength;
} IFDCard;


RESPONSECODE IFDHCreateChannel ( DWORD Lun, DWORD Channel ) {

  RESPONSECODE lRetVal;
  char portID[50];

  /* Time to use new method of port type */

  sprintf( portID, "%s%d", "/dev/pcsc/", Channel );

#ifdef PCSC_DEBUG
  printf("Initializing reader at: %s\n", portID);
#endif

  lRetVal = CMAN_Initialize( portID );
  
  if ( lRetVal == STATUS_SUCCESS ) {
    return IFD_SUCCESS;
  } else {
    return IFD_COMMUNICATION_ERROR;
  }  
}

RESPONSECODE IFDHCloseChannel ( DWORD Lun ) {
  
  RESPONSECODE lRetVal;

  lRetVal = CMAN_UnInitialize( );
  
  if ( lRetVal == STATUS_SUCCESS ) {
    return IFD_SUCCESS;
  } else {
    return IFD_COMMUNICATION_ERROR;
  }    
}

RESPONSECODE IFDHGetCapabilities ( DWORD Lun, DWORD Tag, 
				   PDWORD Length, PUCHAR Value ) {
  
  switch ( Tag ) {

  case TAG_IFD_ATR:
    memcpy( Value, IFDCard.Atr, IFDCard.AtrLength );
    *Length = IFDCard.AtrLength;
    break;

  default:
    return IFD_ERROR_TAG;
    
  }
   
  return IFD_SUCCESS;
}

RESPONSECODE IFDHSetCapabilities ( DWORD Lun, DWORD Tag, 
			       DWORD Length, PUCHAR Value ) {

  /* Not used by the resource manager currently */
  return IFD_SUCCESS; 
}

RESPONSECODE IFDHSetProtocolParameters ( DWORD Lun, DWORD Protocol, 
				   UCHAR Flags, UCHAR PTS1,
				   UCHAR PTS2, UCHAR PTS3) {

  /* This function should set the PTS of a particular card/slot using
     the three PTS parameters sent

     Protocol  - 0 .... 14  T=0 .... T=14
     Flags     - Logical OR of possible values:
     IFD_NEGOTIATE_PTS1 IFD_NEGOTIATE_PTS2 IFD_NEGOTIATE_PTS3
     to determine which PTS values to negotiate.
     PTS1,PTS2,PTS3 - PTS Values.

     returns:

     IFD_SUCCESS
     IFD_ERROR_PTS_FAILURE
     IFD_COMMUNICATION_ERROR
     IFD_PROTOCOL_NOT_SUPPORTED
  */

  return IFD_SUCCESS;
}


RESPONSECODE IFDHPowerICC ( DWORD Lun, DWORD Action, 
			    PUCHAR Atr, PDWORD AtrLength ) {

  RESPONSECODE lRetVal;
  BYTE pbAtr[MAX_ATR_SIZE];
  int iAtrLen;
  int i;

  if ( Action == IFD_POWER_UP ) {
    lRetVal = CMAN_Reset(pbAtr, &iAtrLen);
    
    if ( lRetVal == STATUS_SUCCESS ) {
      memcpy( IFDCard.Atr, pbAtr, iAtrLen );      /* Copies the ATR.       */
      memcpy( Atr, pbAtr, iAtrLen );
      IFDCard.AtrLength = iAtrLen;
      *AtrLength = iAtrLen;
    }
 
  } else if ( Action == IFD_POWER_DOWN ) {
    /* Do Nothing for now */
    
  } else if ( Action == IFD_RESET ) {
    lRetVal = CMAN_Reset(pbAtr, &iAtrLen);
    
    if ( lRetVal == STATUS_SUCCESS ) {
      memcpy( IFDCard.Atr, pbAtr, iAtrLen );      /* Copies the ATR.       */
      memcpy( Atr, pbAtr, iAtrLen );
      IFDCard.AtrLength = iAtrLen;
      *AtrLength = iAtrLen;
    }

  } else {
    /* Unsupported Tag */
    return IFD_NOT_SUPPORTED;
  }

  if ( lRetVal == STATUS_SUCCESS ) {
    return IFD_SUCCESS;
  } else {
    return IFD_ERROR_POWER_ACTION;
  }
}

RESPONSECODE IFDHTransmitToICC ( DWORD Lun, SCARD_IO_HEADER SendPci, 
				 PUCHAR TxBuffer, DWORD TxLength, 
				 PUCHAR RxBuffer, PDWORD RxLength, 
				 PSCARD_IO_HEADER RecvPci ) {
  
  RESPONSECODE lRetVal;
  int i;
  int iRxSize;

  i       = 0;
  iRxSize = 0;

  /* Print Transmit Error Messaging */
  
#ifdef PCSC_DEBUG
  printf("[%04x] -> ", TxLength );
  
  for ( i=0; i < TxLength; i++ ) {
    printf("%02x ", TxBuffer[i]);
  } printf("\n");
#endif  
  
  if ( SendPci.Protocol == 0 ) {    
    lRetVal = CMAN_TransmitT0( TxLength, TxBuffer, &iRxSize, 
			       RxBuffer );
    *RxLength = iRxSize;
    
  } else if ( SendPci.Protocol == 1 ) {
    lRetVal = T1_ExchangeData( TxBuffer, TxLength, RxBuffer, 
			       &iRxSize );
    *RxLength = iRxSize;

  } else {
    return IFD_PROTOCOL_NOT_SUPPORTED;
  }
  
  /* Bale out if it didn't work    */
  if ( lRetVal != STATUS_SUCCESS ) {
    *RxLength = 0;
    return IFD_COMMUNICATION_ERROR;
  }
  
  /* Print Receive Error Messaging */
  
#ifdef PCSC_DEBUG
  printf("[%04x] <- ", *RxLength );
  
  for ( i=0; i < *RxLength; i++ ) {
    printf("%02x ", RxBuffer[i]);
  } printf("\n");
#endif  
  
  return IFD_SUCCESS;
}

RESPONSECODE IFDHControl ( DWORD Lun, PUCHAR TxBuffer, 
			 DWORD TxLength, PUCHAR RxBuffer, 
			 PDWORD RxLength ) {

  /* This function performs a data exchange with the reader (not the card)
     specified by Lun.  Here XXXX will only be used.
     It is responsible for abstracting functionality such as PIN pads,
     biometrics, LCD panels, etc.  You should follow the MCT, CTBCS 
     specifications for a list of accepted commands to implement.

     TxBuffer - Transmit data
     TxLength - Length of this buffer.
     RxBuffer - Receive data
     RxLength - Length of the received data.  This function will be passed
     the length of the buffer RxBuffer and it must set this to the length
     of the received data.

     Notes:
     RxLength should be zero on error.
  */

  return IFD_SUCCESS;
}

RESPONSECODE IFDHICCPresence( DWORD Lun ) {

  /* This function returns the status of the card inserted in the 
     reader/slot specified by Lun.  It will return either:

     returns:
     IFD_ICC_PRESENT
     IFD_ICC_NOT_PRESENT
     IFD_COMMUNICATION_ERROR
  */

 RESPONSECODE lRetVal;
  
 checkcardstate:
  
  lRetVal = CMAN_IsCardInserted();
  
  if ( lRetVal == STATUS_SUCCESS ) {
    return IFD_ICC_PRESENT;
  } else if ( lRetVal == STATUS_UNSUCCESSFUL ) {
    return IFD_ICC_NOT_PRESENT;
  } else if ( lRetVal == STATUS_IO_TIMEOUT ) {    
    CMAN_ResynchReader();
    goto checkcardstate;
  } else {
    printf("problem %d\n", lRetVal);
    return IFD_COMMUNICATION_ERROR;
  }
}
