/*****************************************************************
/
/ File   :   serial.c
/ Author :   David Corcoran
/            -modified by jouni.rosenlof@pp.htv.fi(KPMG IRM Finland)
/ Date   :   March 2000
/ Purpose:   Serial IO functions.
/ License:   See file LICENSE
********************************************************************/

#include <stdio.h>                       /* Standard Includes     */
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <assert.h>

#include <serial.h>
                                        /* For Unix Support      */
#include <termios.h>
#include <sys/signal.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <pcscdefines.h>
#include <LinuxDefines.h>

struct IO_Specs {
  int handle;
  BYTE   baud;
  BYTE   bits;
  char   parity;
  long   blocktime;
} ioport;


/* Delay for 5 ms */
static void Delay ()
{
  /* tstruct contains the delay value (sec,usec); default 5ms */
  struct timeval tstruct;
  tstruct.tv_sec  = 0;
  tstruct.tv_usec = 1000;
  
  select (1, NULL, NULL, NULL, &tstruct);
}


/*
 * InititalizePort -- initialize a serial port
 *	This functions opens the serial port specified and sets the
 *	line parameters baudrate, bits per byte and parity according to the
 *	parameters.
 */

int
IO_InitializePort(int baud, int bits, char parity, char* port)
{
  /* UNIX SERIAL SUPPORT */
  
  int handle; 
  struct termios tstr;
  char pcError[70];
  int status;	
  
  handle = open(port, O_RDWR | O_NOCTTY);	
  
  if (handle < 0) {	/* Problems with /dev/smartcard  */
    sprintf(pcError, "Cannot open port %s", port);
    perror(pcError);
    return FALSE;
  }
  
  memset ((void*) &tstr, 0, sizeof (tstr)); 
  
  /* 
     PARMRK Mark parity errors. 
     INPCK  Enable input parity check. 
  */
//  tstr.c_iflag = PARMRK | INPCK;
  tstr.c_iflag = (tcflag_t) NULL;
  tstr.c_oflag = (tcflag_t) NULL;
  //  tstr.c_iflag |= BRKINT;
  /* 
     B9600               9600 baud 
     PARENB Parity enable   (even by default) 
     CS8  8 bits            (one stop bit by default) 
     CREAD  Enable receiver 
     CLOCAL Local line, else dial-up 
  */
  tstr.c_cflag = B9600 | PARENB | CS8 | CREAD | CLOCAL;
  //tstr.c_lflag = NOFLSH;
  tstr.c_cc[VMIN] = 0;		/* Minimum number of characters = 0 */
  tstr.c_cc[VTIME] = 5;		/* Timeout value in .1 sec intervals = 5 */
  
  /* Set the serial port parameters */
  status = ioctl (handle, TCSETS, &tstr);
  if (status)
    perror ("Set params:");
  
  
  ioport.handle = handle;            /* Record the handle                 */
  ioport.baud   = baud;              /* Record the baudrate               */
  ioport.bits   = bits;              /* Record the bits                   */
  ioport.parity = parity;            /* Record the parity                 */
  ioport.blocktime = 0;              /* Default Beginning Blocktime       */

  return TRUE;
}

int
IO_CardManturnon () {
	return TRUE;
}

int
IO_CardManpulserts () {

	int handle;
	int modem_control;
	int status;
	
	handle = IO_ReturnHandle ();
	modem_control = TIOCM_RTS;
	status = ioctl (handle, TIOCMBIC, &modem_control);   /* Reset Low  */
	if (status) {
		perror ("Reset Low:");
		return FALSE;
	}
	
	Delay ();
	
	status = ioctl (handle, TIOCMBIS, &modem_control);   /* Reset High */
       	if (status) {
		perror ("Reset High:");
		return FALSE;
	}
	
	return TRUE;
}


int
IO_ReturnHandle() {
  return ioport.handle;       /* Return the current used handle      */
}

int 
IO_UpdateReturnBlock(int blocktime) {    
  /* Sets the blocking timeout value, in microseconds                */
  ioport.blocktime = (long)blocktime;
  return ioport.blocktime;   
}

int
IO_ReturnBaudRate() {
  return ioport.baud;         /* Return the current baudrate         */
}

int
IO_FlushBuffer() {
  
  int handle = ioport.handle;

  if (tcflush(handle, TCIFLUSH) == 0)
    return TRUE;
  return FALSE;

}


/* IO_Read:
	Reads a string of bytes of a known length.
	Used basicly in communication with the reader
*/
int
IO_Read( int readsize, BYTE *response ) { 
  
  fd_set rfds;
  struct timeval tv;
  int rval;
  BYTE c;
  int handle;
  int count = 0;
  
  handle = ioport.handle;

  tv.tv_sec  = 0;
  tv.tv_sec  = ioport.blocktime;
  tv.tv_usec = 600000;

  FD_ZERO(&rfds);

#ifdef PCSC_DEBUG
  printf("<- "); 
#endif
 
  for (count=0; count < readsize; count++) {
    FD_SET(handle, &rfds);
    rval = select (handle+1, &rfds, NULL, NULL, &tv);
    
    if (FD_ISSET(handle, &rfds)) {
      read(handle, &c, 1);

#ifdef PCSC_DEBUG
      printf("%02x ", c);
#endif

      response[count] = c;
    }
    else {
      tcflush(handle, TCIFLUSH);

#ifdef PCSC_DEBUG
      printf("\n");
#endif

      return FALSE;
    }
  }

#ifdef PCSC_DEBUG
  printf("\n");
#endif

  return TRUE;
  
}

int IO_Write(BYTE *c, int writesize) {

  int handle = ioport.handle;
  int i;

#ifdef PCSC_DEBUG
  printf("-> ");
#endif

  for (i=0; i < writesize; i++) {
    if (write(handle, &c[i], 1) == 1) {            /* Write one byte   */
#ifdef PCSC_DEBUG
      printf("%02x ", c[i]);
#endif
    } else {
#ifdef PCSC_DEBUG
      printf("\n");
#endif
      return FALSE;
    }
  }

#ifdef PCSC_DEBUG
  printf("\n");
#endif

  return TRUE;
}


int IO_Close() {

  int handle = ioport.handle;

  if ( close (handle) == 0 ) {                      /* Close the handle */
    return TRUE;
  } else {
    return FALSE;                                  /* Done !            */
  }

}
